/* libopt++/src/libopt++/Parser.cpp
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace opt {



Parser::Parser()
    : progName(_progName), progPath(_progPath), progDir(_progDir), args(_args)
{
}



void Parser::addOption(int callbackResult, bool arg, const std::string& opt, char shopt,
                       const std::string& help)
{
    if(!opt.size() && !shopt) throw std::invalid_argument("opt, shopt");
    opts.push_back(Opt(callbackResult, arg, opt, shopt, help));
}



void Parser::parse(int argc, const char* const* argv)
{
    bool parseopts = true, expectingArg = false;
    int whichOpt = -1;
    std::string opt;

    if(argc < 1 || !argv) throw std::invalid_argument("argc/argv[]");

    // get program name
    if(!argv[0]) throw std::invalid_argument("argv[0]");
    _progPath = argv[0];
    std::string::size_type pos = _progPath.rfind('/');
    if(pos != std::string::npos) {
        _progName = _progPath.substr(pos + 1, std::string::npos);
        _progDir = _progPath.substr(0, pos ?: 1);
    } else {
        _progName = _progPath;
    }

    // interpret remaining arguments / options
    for(int i = 1; i < argc; ++i) {
        const char* arg = argv[i];
        if(!arg) throw std::invalid_argument("argv[]");

        if(parseopts && arg[0] && arg[0] == '-' && arg[1]) {
            if(expectingArg) goto missingArg;

            if(arg[1] == '-') {
                // long option
                if(!arg[2]) parseopts = false; // -- switches off option parsing
                else {
                    opt = arg;
                    whichOpt = findOpt(opt);
                    if(opts.at(whichOpt).arg) expectingArg = true;
                    else option(opts.at(whichOpt).result, opt);
                }
            } else {
                // short option[s]
                for(const char* p = arg + 1; *p; ++p) {
                    if(expectingArg) goto missingArg;
                    opt = "-x";
                    opt[1] = *p;
                    whichOpt = findOpt(opt);
                    if(opts.at(whichOpt).arg) expectingArg = true;
                    else option(opts.at(whichOpt).result, opt);
                }
            }
        } else {
            if(expectingArg) {
                // option argument
                expectingArg = false;
                option(opts.at(whichOpt).result, opt, arg);
            } else {
                // program argument
                _args.push_back(arg);
            }
        }
    }

    if(!expectingArg) return;

missingArg:
    std::ostringstream str;
    str << "Option '" << opt << "' is missing an argument.";
    throw BadArg(str.str());
}



void Parser::showHelp(std::ostream& str) const
{
    const uint end = opts.size();
    std::string::size_type max = 0;

    char oldfill = str.fill(' ');
    std::ios_base::fmtflags oldfmt = str.setf(std::ios_base::left, std::ios_base::adjustfield);

    for(uint i = 0; i < end; ++i) {
        if(!opts[i].opt.size()) continue;
        max = std::max(max, opts[i].opt.size() + (opts[i].arg ? 6 : 0));
    }

    for(uint i = 0, end = opts.size(); i < end; ++i) {
        str << "    ";
        if(opts[i].shopt) {
            str << '-' << opts[i].shopt;
            if(opts[i].opt.size()) {
                str << ", --" << std::setw(max - (opts[i].arg ? 6 : 0)) << opts[i].opt;
                if(opts[i].arg) str << " <arg>";
            } else {
                if(opts[i].arg) str << " <arg>    " << std::setw(max - 6) << ' ';
                else str << std::setw(max) << ' ';
            }
        } else {
            str << "    --" << std::setw(max - (opts[i].arg ? 6 : 0)) << opts[i].opt;
            if(opts[i].arg) str << " <arg>";
        }

        str << "    " << opts[i].help << '\n';
    }
    str << "    Short options without arguments may be combined. Pass -- to\n"
        "    stop processing options.\n";

    str.fill(oldfill);
    str.setf(oldfmt, std::ios_base::adjustfield);
}



uint Parser::findOpt(const std::string& opt)
{
    const uint end = opts.size();

    if(opt[1] == '-') {
        std::string opt2 = opt.substr(2, std::string::npos);
        for(uint pos = 0; pos < end; ++pos) if(opts[pos].opt == opt2) return pos;
    } else {
        for(uint pos = 0; pos < end; ++pos) if(opts[pos].shopt == opt[1]) return pos;
    }

    std::ostringstream str;
    str << "Unrecognsied option '" << opt << "'.";
    throw BadArg(str.str());
}



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
