/* libopt++/src/libopt++/Parser.h
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

namespace opt {



/*! \brief Main option parser.

This class is responsible for parsing commandline options. It is an abstract base class; you must
derive from it and overload the option() functions. These specify the behaviour on encountering
options.

To use the class, you must first instantiate it and then call addOption() for each option your
program understands. Once this is complete, you can call parse() with the argc/argv that were passed
to your program's main().

parse() will set \ref progName, \ref progPath and \ref progDir appropriately, and will call option()
for each valid option it encounters. It will store any non-option arguments in the \ref args vector.

*/
class Parser {
public:
    /// Constructor (does nothing special).
    Parser();

    /// Destructor (does nothing special).
    virtual ~Parser() throw() { }



    /*! \brief Registers an option that the program understands.

    \param callbackResult This is what is passed to option() as the \a result argument.
    \param arg \a true if the option requires an argument, \a false if not.
    \param opt The options long name, excluding leading \c --. May be empty.
    \param shopt The short option character (used in e.g. \c -v). May be 0 (no short option).
    \param help A string displayed when showHelp() is called. May be empty.
    \throws std::invalid_argument if \a opt is empty and \a shopt is 0.

    This registers an option that the program understands with the parser engine. Each option has
    a corresponding code, \a callbackResult, that is used to uniquely identify it. This is parsed
    to the option() callback as the first argument, \a result. An option may have both short and
    long forms.

    */
    void addOption(int callbackResult, bool arg, const std::string& opt, char shopt = 0,
                   const std::string& help = "");



    /*! \brief Parse commandline options.

    \param argc argc from main(), the argument count.
    \param argv argv from main(), the argument vector.
    \throws std::invalid_argument if argc or argv are invalid in some way (this won't happen if you
        pass them directly from main).
    \throws Opt::BadArg if an option is unrecognised or missing an argument.

    */
    void parse(int argc, const char* const* argv);



    /*! \brief Display help.

    \param ostr The output stream to display help onto (typically \a std::cout).

    This function displays a nicely-formatted list of options to be used as part of a help screen.
    It uses the \a help string passed to addOption() to display information about each option.

    */
    void showHelp(std::ostream& ostr) const;



    /// The program name (filename only, no path).
    const std::string& progName;

    /// The program path (full filename and path, i.e. argv[0]).
    const std::string& progPath;

    /// The program directory (relative, without filename).
    const std::string& progDir;

    /// A list of the arguments (non-option items on the commandline).
    const std::vector<std::string>& args;



    /*! \brief Option callback.

    \param result The result code given as \a callbackResult in addOption().
    \param opt The formatted option name (i.e. "-x" or "--long").

    This function is the callback for options encountered in parse(). This version does not take
    any arguments.

    */
    virtual void option(int result, const std::string& opt) = 0;

    /*! \brief Option callback.

    \param result The result code given as \a callbackResult in addOption().
    \param opt The formatted option name (i.e. "-x" or "--long").
    \param arg The argument following the option.

    This function is the callback for options encountered in parse(). This version takes a
    mandatory argument.

    */
    virtual void option(int result, const std::string& opt, const std::string& arg) = 0;

private:
    std::string _progName, _progPath, _progDir;
    std::vector<std::string> _args;

    struct Opt {
        int result;
        bool arg;
        std::string opt;
        char shopt;
        std::string help;
        Opt(int result, bool arg, const std::string& opt, char shopt, const std::string& help)
            : result(result), arg(arg), opt(opt), shopt(shopt), help(help)
        { }
    };
    std::vector<Opt> opts;
    uint findOpt(const std::string& opt);
};



}

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
